@@ -1,7 +1,7 @@
 /* Global declarations for the ed editor.  */
 /* GNU ed - The GNU line editor.
    Copyright (C) 1993, 1994 Andrew L. Moore, Talke Studio
-   Copyright (C) 2006-2023 Antonio Diaz Diaz.
+   Copyright (C) 2006-2025 Antonio Diaz Diaz.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,10 +17,7 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#ifndef __cplusplus
-enum Bool { false = 0, true = 1 };
-typedef enum Bool bool;
-#endif
+#include <stdbool.h>
 
 enum Pflags			/* print suffixes */
   {
@@ -75,6 +72,7 @@ bool isbinary( void );
 bool join_lines( const int from, const int to, const bool isglobal );
 int last_addr( void );
 bool modified( void );
+bool warned( void );
 bool move_lines( const int first_addr, const int second_addr, const int addr,
                  const bool isglobal );
 bool open_sbuf( void );
@@ -84,7 +82,8 @@ const char * put_sbuf_line( const char * const buf, const int size );
 line_t * search_line_node( const int addr );
 void set_binary( void );
 void set_current_addr( const int addr );
-void set_modified( const bool m );
+void set_modified( const bool b );
+void set_warned( const bool b );
 bool yank_lines( const int from, const int to );
 void clear_undo_stack( void );
 undo_t * push_undo_atom( const int type, const int from, const int to );
@@ -103,7 +102,8 @@ bool get_extended_line( const char ** const ibufpp, int * const lenp,
 const char * get_stdin_line( int * const sizep );
 int linenum( void );
 bool print_lines( int from, const int to, const int pflags );
-int read_file( const char * const filename, const int addr );
+int read_file( const char * const filename, const int addr,
+               bool * const read_onlyp );
 int write_file( const char * const filename, const char * const mode,
                 const int from, const int to );
 void reset_unterminated_line( void );
@@ -113,13 +113,17 @@ void unmark_unterminated_line( const line_t * const lp );
 bool extended_regexp( void );
 bool interactive();
 bool may_access_filename( const char * const name );
+void print_filename( const char * const filename, const bool to_stdout );
 bool restricted( void );
 bool scripted( void );
 void show_strerror( const char * const filename, const int errcode );
+void show_warning( const char * const filename, const char * const msg );
 bool strip_cr( void );
 bool traditional( void );
 
 /* defined in main_loop.c */
+const char * error_msg( void );
+int first_e_command( const char * const filename );
 void invalid_address( void );
 int main_loop( const bool initial_error, const bool loose );
 bool set_def_filename( const char * const s );
@@ -143,6 +147,7 @@ bool subst_regex( void );
 /* defined in signal.c */
 void disable_interrupts( void );
 void enable_interrupts( void );
+const char * home_directory( void );
 bool resize_buffer( char ** const buf, int * const size, const unsigned min_size );
 void set_signals( void );
 void set_window_lines( const int lines );
